# Generated by Django 3.0.3 on 2020-06-16 10:20

import core.fields
import datetime
from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('policy', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='PolicyRenewal',
            fields=[
                ('validity_from', core.fields.DateTimeField(db_column='ValidityFrom', default=datetime.datetime.now)),
                ('validity_to', core.fields.DateTimeField(blank=True, db_column='ValidityTo', null=True)),
                ('legacy_id', models.IntegerField(blank=True, db_column='LegacyID', null=True)),
                ('id', models.AutoField(db_column='RenewalID', primary_key=True, serialize=False)),
                ('uuid', models.CharField(db_column='RenewalUUID', default=uuid.uuid4, max_length=36, unique=True)),
                ('renewal_prompt_date', core.fields.DateField(db_column='RenewalPromptDate')),
                ('renewal_date', core.fields.DateField(db_column='RenewalDate')),
                ('phone_number', models.CharField(blank=True, db_column='PhoneNumber', max_length=25, null=True)),
                ('sms_status', models.SmallIntegerField(db_column='SMSStatus', default=0)),
                ('renewal_warnings', models.SmallIntegerField(blank=True, db_column='RenewalWarnings', default=0, null=True)),
                ('response_status', models.IntegerField(blank=True, db_column='ResponseStatus', default=0, null=True)),
                ('response_date', core.fields.DateTimeField(blank=True, db_column='ResponseDate', null=True)),
                ('audit_user_id', models.IntegerField(blank=True, db_column='AuditCreateUser', null=True)),
            ],
            options={
                'db_table': 'tblPolicyRenewals',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='PolicyRenewalDetail',
            fields=[
                ('validity_from', core.fields.DateTimeField(db_column='ValidityFrom', default=datetime.datetime.now)),
                ('validity_to', core.fields.DateTimeField(blank=True, db_column='ValidityTo', null=True)),
                ('legacy_id', models.IntegerField(blank=True, db_column='LegacyID', null=True)),
                ('id', models.AutoField(db_column='RenewalDetailID', primary_key=True, serialize=False)),
                ('audit_user_id', models.IntegerField(blank=True, db_column='AuditCreateUser', null=True)),
            ],
            options={
                'db_table': 'tblPolicyRenewalDetails',
                'managed': False,
            },
        ),
    ]
