# Generated by Django 3.2.18 on 2023-05-12 11:34

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('policy', '0005_add_foreign_keys'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='policy',
            options={'managed': True},
        ),
        migrations.AlterModelOptions(
            name='policyrenewal',
            options={'managed': True},
        ),
    ]