# Generated by Django 3.2.19 on 2023-07-12 17:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('policy', '0006_set_model_managed_to_true'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='policymutation',
            table='policy_PolicyMutation',
        ),
    ]
