# Generated by Django 3.0.3 on 2020-10-21 08:11

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0009_mutationlog_client_mutation_details'),
        ('policy', '0002_policy_renewals'),
    ]

    operations = [
        migrations.CreateModel(
            name='PolicyMutation',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('mutation', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='policies', to='core.MutationLog')),
                ('policy', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='mutations', to='policy.Policy')),
            ],
            options={
                'db_table': 'location_PolicyMutation',
                'managed': True,
            },
        ),
        migrations.DeleteModel(
            name='PolicyRenewalDetail',
        ),
    ]
